@@ -2,13 +2,17 @@
 
 from __future__ import annotations
 
+import io
+import functools
 import re
 from dataclasses import dataclass
 from html.parser import HTMLParser
 from pathlib import Path
-from typing import Iterable, Iterator, List, Mapping, Optional, Sequence
+from typing import Callable, Iterable, Iterator, List, Mapping, Optional, Sequence
 
+from docutils import nodes
 from docutils.core import publish_parts
+from docutils.parsers.rst import Directive, directives, roles
 from docutils.utils import SystemMessage
 
 
@@ -71,7 +75,12 @@ class DocumentIndex:
             project_root = roots_or_project.resolve()
             default_doc_path = doc_path.resolve() if doc_path is not None else (project_root / "docs" / "source")
             self._roots = (default_doc_path,)
-            alias_input = aliases or {default_doc_path: "docs/source"}
+            alias_value = None
+            if aliases is not None:
+                alias_input = aliases
+            else:
+                alias_value = _default_alias(default_doc_path)
+                alias_input = {default_doc_path: alias_value} if alias_value is not None else {default_doc_path: ""}
         else:
             resolved_roots = tuple(sorted(Path(root).resolve() for root in roots_or_project))
             if not resolved_roots:
@@ -86,14 +95,18 @@ class DocumentIndex:
             missing_str = ", ".join(str(root) for root in missing_roots)
             raise FileNotFoundError(f"Documentation root(s) not found: {missing_str}")
 
-        alias_map: dict[Path, str] = {
-            Path(key).resolve(): value.strip("/")
-            for key, value in alias_input.items()
-        }
+        alias_map: dict[Path, str] = {}
+        for key, value in alias_input.items():
+            resolved = Path(key).resolve()
+            alias_map[resolved] = value.strip("/") if value else ""
         self._aliases = {root: alias_map.get(root, "") for root in self._roots}
 
         self._include_extensions = tuple(sorted(ext.lower() for ext in include_extensions))
-        self._extra_files = {}
+        self._extra_files: dict[Path, str | None] = {}
+        if extra_files:
+            for file_path, doc_id in extra_files.items():
+                resolved = Path(file_path).resolve()
+                self._extra_files[resolved] = doc_id
         self._documents: dict[str, DocumentRecord] = {}
         self.refresh()
 
@@ -109,6 +122,12 @@ class DocumentIndex:
             doc_id = self._doc_id_for_path(root=root, path=path)
             documents[doc_id] = _build_record(doc_id, path)
 
+        for path, explicit_id in self._extra_files.items():
+            if not path.exists() or path.suffix.lower() not in self._include_extensions:
+                continue
+            doc_id = explicit_id or path.name
+            documents[doc_id] = _build_record(doc_id, path)
+
         self._documents = documents
 
     def list_documents(self) -> list[DocumentRecord]:
@@ -363,14 +382,27 @@ class _RSTHTMLToTextParser(HTMLParser):
 
 
 def _parse_rst_document(source: str, path: Path) -> tuple[str, str, list[DocumentSection]]:
+    _prepare_rst_environment()
+    sanitized_source = _strip_sphinx_roles(source)
     try:
-        parts = publish_parts(source=source, writer_name="html")
+        warning_buffer = io.StringIO()
+        parts = publish_parts(
+            source=sanitized_source,
+            source_path=str(path),
+            writer_name="html",
+            settings_overrides={
+                "report_level": 5,
+                "halt_level": 6,
+                "exit_status_level": 6,
+                "warning_stream": warning_buffer,
+            },
+        )
         title = parts.get("title")
         body = parts.get("body", "")
     except (SystemMessage, ImportError):
-        sections = list(_extract_sections(source))
+        sections = list(_extract_sections(sanitized_source))
         title = sections[0].title if sections else path.stem.replace("_", " ").title()
-        return title, source, sections
+        return title, sanitized_source, sections
 
     parser = _RSTHTMLToTextParser()
     parser.feed(body)
@@ -426,6 +458,95 @@ def _parse_rst_document(source: str, path: Path) -> tuple[str, str, list[Documen
     return resolved_title, plain_text, sections
 
 
+def _strip_sphinx_roles(source: str) -> str:
+    # Remove interpreted text roles such as ``:bdg:`value``` to leave only the payload.
+    source = re.sub(r"\:([\w.+-]+)\:`([^`]+)`", r"\2", source)
+    # Replace Sphinx-style references ``Foo`_``/``Foo`__`` with plain text.
+    source = re.sub(r"`([^`]+)`__?", r"\1", source)
+    return source
+
+
+@functools.lru_cache(maxsize=1)
+def _prepare_rst_environment() -> None:
+    class _PassthroughDirective(Directive):
+        has_content = True
+        optional_arguments = 1
+        final_argument_whitespace = True
+        option_spec: dict[str, Callable[[str], str]] = {}
+
+        def run(self):
+            container = nodes.container()
+            if self.content:
+                self.state.nested_parse(self.content, self.content_offset, container)
+            return list(container.children)
+
+    class _IgnoreDirective(Directive):
+        has_content = True
+        optional_arguments = 0
+
+        def run(self):
+            return []
+
+    class _LiteralIncludeDirective(Directive):
+        required_arguments = 1
+        optional_arguments = 0
+        final_argument_whitespace = False
+        has_content = False
+        option_spec = {
+            "language": directives.unchanged,
+            "linenos": directives.flag,
+        }
+
+        def run(self):
+            argument = self.arguments[0]
+            language = self.options.get("language")
+            show_linenos = "linenos" in self.options
+
+            settings = self.state.document.settings
+            source_candidate = (
+                getattr(settings, "_source", None)
+                or getattr(settings, "_source_path", None)
+                or self.state.document.current_source
+            )
+            base_path = Path(source_candidate or ".").resolve().parent
+            include_path = (base_path / argument).resolve()
+            try:
+                text = include_path.read_text(encoding="utf-8")
+            except OSError:
+                text = ""
+            literal = nodes.literal_block(text, text)
+            if language:
+                literal["language"] = language
+            if show_linenos:
+                literal["linenos"] = True
+            return [literal]
+
+    directives.register_directive("dropdown", _PassthroughDirective)
+    directives.register_directive("tab-set", _PassthroughDirective)
+    directives.register_directive("tab-item", _PassthroughDirective)
+    directives.register_directive("toctree", _IgnoreDirective)
+    directives.register_directive("literalinclude", _LiteralIncludeDirective)
+
+    def _noop_role(role_name, rawtext, text, lineno, inliner, options=None, content=None):
+        return [nodes.Text(text)], []
+
+    for role_name in (
+        "bdg",
+        "bdg-dark-line",
+        "bdg-danger",
+        "bdg-secondary",
+        "ref",
+        "doc",
+        "class",
+        "mod",
+        "func",
+        "meth",
+        "data",
+        "program",
+    ):
+        roles.register_local_role(role_name, _noop_role)
+
+
 def _normalize_anchor(value: str) -> str:
     normalized = re.sub(r"[^0-9a-zA-Z]+", "-", value.strip().lower()).strip("-")
     return normalized or "section"
@@ -457,3 +578,12 @@ def _heading_for_line(sections: Sequence[DocumentSection], line_number: int) ->
         elif section.start_line > line_number:
             break
     return match
+
+
+def _default_alias(path: Path) -> str | None:
+    parts = path.parts
+    if len(parts) >= 2 and parts[-2] == "docs" and parts[-1] == "md":
+        return "docs/md"
+    if len(parts) >= 2 and parts[-2] == "docs" and parts[-1] == "source":
+        return "docs/source"
+    return path.name if path.name else None
