@@ -15,7 +15,9 @@ from flask import Flask, current_app
 from jinja2 import Environment, FileSystemLoader
 
 from flarchitect.utils.config_helpers import get_config_or_model_meta
-from flarchitect.utils.core_utils import convert_case, get_count
+from importlib import import_module
+
+from flarchitect.utils.core_utils import get_count
 
 HTTP_METHODS = ["GET", "POST", "PATCH", "DELETE"]
 DATE_FORMAT = "%Y-%m-%d"
@@ -375,7 +377,13 @@ def pretty_print_dict(d: dict[Any, Any]) -> str:
     return pprint.pformat(d, indent=2)
 
 
-def update_dict_if_flag_true(output: dict[str, Any], flag: bool, key: str, value: Any, case_func: Any) -> None:
+def update_dict_if_flag_true(
+    output: dict[str, Any],
+    flag: bool,
+    key: str,
+    value: Any,
+    case_func: str | Callable[[str], str],
+) -> None:
     """Update a dictionary with a key-value pair if the flag is True.
 
     Args:
@@ -385,8 +393,16 @@ def update_dict_if_flag_true(output: dict[str, Any], flag: bool, key: str, value
         value (Any): The value to associate with the key.
         case_func (Any): The function to convert the case of the key.
     """
-    if flag:
-        output.update({convert_case(key, case_func): value})
+    if not flag:
+        return
+
+    converted: str
+    if callable(case_func):
+        converted = case_func(key)
+    else:
+        converted = _convert_key_case(key, str(case_func)) if case_func else key
+
+    output[converted] = value
 
 
 def make_base_dict() -> dict[str, Any]:
@@ -421,6 +437,16 @@ def make_base_dict() -> dict[str, Any]:
     return output
 
 
+def _convert_key_case(key: str, target_case: str) -> str:
+    """Resolve ``convert_case`` lazily to avoid stale stubs during testing."""
+
+    module = import_module("flarchitect.utils.core_utils")
+    converter = getattr(module, "convert_case", None)
+    if callable(converter):
+        return converter(key, target_case)
+    return key
+
+
 def pluralize_last_word(converted_name: str) -> str:
     """
     Pluralize the last word of the converted name
@@ -546,5 +572,6 @@ def _looks_like_response_container(result: Any) -> bool:
 HTTP_OK = 200
 HTTP_BAD_REQUEST = 400
 HTTP_INTERNAL_SERVER_ERROR = 500
+HTTP_UNPROCESSABLE_ENTITY = 422
 HTTP_UNAUTHORIZED = 401
 HTTP_FORBIDDEN = 403
